@@ -31,7 +31,6 @@
 #    this exception statement from your version. If you delete this exception
 #    statement from all source files in the program, then also delete it here.
 #
-
 #
 
 
@@ -78,6 +77,24 @@ def filter_one_keyword(torrent_ids, keyword):
                     yield torrent_id
                     break
 
+def tracker_error_filter(torrent_ids, values):
+    filtered_torrent_ids = []
+    tm = component.get("TorrentManager")
+
+    # If this is a tracker_host, then we need to filter on it
+    if values[0] != "Error":
+        for torrent_id in torrent_ids:
+            if values[0] == tm[torrent_id].get_status(["tracker_host"])["tracker_host"]:
+                filtered_torrent_ids.append(torrent_id)
+        return filtered_torrent_ids
+
+    # Check all the torrent's tracker_status for 'Error:' and only return torrent_ids
+    # that have this substring in their tracker_status
+    for torrent_id in torrent_ids:
+        if "Error:" in tm[torrent_id].get_status(["tracker_status"])["tracker_status"]:
+            filtered_torrent_ids.append(torrent_id)
+
+    return filtered_torrent_ids
 
 class FilterManager(component.Component):
     """FilterManager
@@ -87,13 +104,17 @@ class FilterManager(component.Component):
         component.Component.__init__(self, "FilterManager")
         log.debug("FilterManager init..")
         self.core = core
-        self.torrents = core.torrents
+        self.torrents = core.torrentmanager
         self.registered_filters = {}
         self.register_filter("keyword", filter_keywords)
         self.tree_fields = {}
 
         self.register_tree_field("state", self._init_state_tree)
-        self.register_tree_field("tracker_host")
+        def _init_tracker_tree():
+            return {"Error": 0}
+        self.register_tree_field("tracker_host", _init_tracker_tree)
+
+        self.register_filter("tracker_host", tracker_error_filter)
 
     def filter_torrent_ids(self, filter_dict):
         """
@@ -119,6 +140,10 @@ class FilterManager(component.Component):
             return torrent_ids
 
         #special purpose: state=Active.
+        if "state" in filter_dict:
+            # We need to make sure this is a list for the logic below
+            filter_dict["state"] = list(filter_dict["state"])
+
         if "state" in filter_dict and "Active" in filter_dict["state"]:
             filter_dict["state"].remove("Active")
             if not filter_dict["state"]:
@@ -141,7 +166,7 @@ class FilterManager(component.Component):
 
         #leftover filter arguments:
         #default filter on status fields.
-        status_func = self.core.export_get_torrent_status #premature optimalisation..
+        status_func = self.core.get_torrent_status #premature optimalisation..
         for torrent_id in list(torrent_ids):
             status = status_func(torrent_id, filter_dict.keys()) #status={key:value}
             for field, values in filter_dict.iteritems():
@@ -156,7 +181,7 @@ class FilterManager(component.Component):
         for use in sidebar.
         """
         torrent_ids = self.torrents.get_torrent_list()
-        status_func = self.core.export_get_torrent_status #premature optimalisation..
+        status_func = self.core.get_torrent_status #premature optimalisation..
         tree_keys = list(self.tree_fields.keys())
         if hide_cat:
             for cat in hide_cat:
@@ -171,6 +196,9 @@ class FilterManager(component.Component):
                 value = status[field]
                 items[field][value] = items[field].get(value, 0) + 1
 
+        if "tracker_host" in items:
+            items["tracker_host"]["Error"] = len(tracker_error_filter(torrent_ids, ("Error",)))
+
         if "state" in tree_keys and not show_zero_hits:
             self._hide_state_items(items["state"])
 
@@ -205,10 +233,11 @@ class FilterManager(component.Component):
         self.tree_fields[field] = init_func
 
     def deregister_tree_field(self, field):
-        del self.tree_fields[field]
+        if field in self.tree_fields:
+            del self.tree_fields[field]
 
     def filter_state_active(self, torrent_ids):
-        get_status = self.core.export_get_torrent_status
+        get_status = self.core.get_torrent_status
         for torrent_id in list(torrent_ids):
             status = get_status(torrent_id, ["download_payload_rate", "upload_payload_rate"])
             if status["download_payload_rate"] or status["upload_payload_rate"]:
@@ -235,6 +264,3 @@ class FilterManager(component.Component):
             iy = 99
 
         return ix - iy
-
-
-
