@@ -31,31 +31,28 @@
 #    this exception statement from your version. If you delete this exception
 #    statement from all source files in the program, then also delete it here.
 #
-
 #
 
 
 """TorrentManager handles Torrent objects"""
 
 import cPickle
-import os.path
 import os
 import time
 import shutil
+import operator
 
-import gobject
+from twisted.internet import reactor
+from twisted.internet.task import LoopingCall
 
-try:
-    import deluge.libtorrent as lt
-except ImportError:
-    import libtorrent as lt
-    if not (lt.version_major == 0 and lt.version_minor == 14):
-        raise ImportError("This version of Deluge requires libtorrent 0.14!")
+from deluge._libtorrent import lt
 
 
+from deluge.event import *
+from deluge.error import *
 import deluge.common
 import deluge.component as component
-from deluge.configmanager import ConfigManager
+from deluge.configmanager import ConfigManager, get_config_dir
 from deluge.core.torrent import Torrent
 from deluge.core.torrent import TorrentOptions
 import deluge.core.oldstateupgrader
@@ -83,6 +80,8 @@ class TorrentState:
             stop_ratio=2.00,
             stop_at_ratio=False,
             remove_at_ratio=False,
+            move_completed=False,
+            move_completed_path=None,
             magnet=None,
             time_added=-1
         ):
@@ -109,26 +108,34 @@ class TorrentState:
         self.stop_ratio = stop_ratio
         self.stop_at_ratio = stop_at_ratio
         self.remove_at_ratio = remove_at_ratio
+        self.move_completed = move_completed
+        self.move_completed_path = move_completed_path
 
 class TorrentManagerState:
     def __init__(self):
         self.torrents = []
 
 class TorrentManager(component.Component):
-    """TorrentManager contains a list of torrents in the current libtorrent
+    """
+    TorrentManager contains a list of torrents in the current libtorrent
     session.  This object is also responsible for saving the state of the
-    session for use on restart."""
+    session for use on restart.
+    """
 
-    def __init__(self, session, alerts):
-        component.Component.__init__(self, "TorrentManager", interval=5000, depend=["PluginManager"])
+    def __init__(self):
+        component.Component.__init__(self, "TorrentManager", interval=5, depend=["CorePluginManager"])
         log.debug("TorrentManager init..")
         # Set the libtorrent session
-        self.session = session
+        self.session = component.get("Core").session
         # Set the alertmanager
-        self.alerts = alerts
+        self.alerts = component.get("AlertManager")
         # Get the core config
         self.config = ConfigManager("core.conf")
 
+        # Make sure the state folder has been created
+        if not os.path.exists(os.path.join(get_config_dir(), "state")):
+            os.makedirs(os.path.join(get_config_dir(), "state"))
+
         # Create the torrents dict { torrent_id: Torrent }
         self.torrents = {}
 
@@ -137,6 +144,12 @@ class TorrentManager(component.Component):
         # and that their resume data has been written.
         self.shutdown_torrent_pause_list = []
 
+        # self.num_resume_data used to save resume_data in bulk
+        self.num_resume_data = 0
+
+        # Keeps track of resume data that needs to be saved to disk
+        self.resume_data = {}
+
         # Register set functions
         self.config.register_set_function("max_connections_per_torrent",
             self.on_set_max_connections_per_torrent)
@@ -158,7 +171,6 @@ class TorrentManager(component.Component):
             self.on_alert_tracker_reply)
         self.alerts.register_handler("tracker_announce_alert",
             self.on_alert_tracker_announce)
-        self.alerts.register_handler("tracker_alert", self.on_alert_tracker)
         self.alerts.register_handler("tracker_warning_alert",
             self.on_alert_tracker_warning)
         self.alerts.register_handler("tracker_error_alert",
@@ -182,9 +194,7 @@ class TorrentManager(component.Component):
 
     def start(self):
         # Get the pluginmanager reference
-        self.plugins = component.get("PluginManager")
-
-        self.signals = component.get("SignalManager")
+        self.plugins = component.get("CorePluginManager")
 
         # Run the old state upgrader before loading state
         deluge.core.oldstateupgrader.OldStateUpgrader()
@@ -193,19 +203,38 @@ class TorrentManager(component.Component):
         self.load_state()
 
         # Save the state every 5 minutes
-        self.save_state_timer = gobject.timeout_add(300000, self.save_state)
-        self.save_resume_data_timer = gobject.timeout_add(290000, self.save_resume_data)
+        self.save_state_timer = LoopingCall(self.save_state)
+        self.save_state_timer.start(200, False)
+        self.save_resume_data_timer = LoopingCall(self.save_resume_data)
+        self.save_resume_data_timer.start(190)
 
     def stop(self):
+        # Stop timers
+        if self.save_state_timer.running:
+            self.save_state_timer.stop()
+
+        if self.save_resume_data_timer.running:
+            self.save_resume_data_timer.stop()
+
         # Save state on shutdown
         self.save_state()
 
+        # Make another list just to make sure all paused torrents will be
+        # passed to self.save_resume_data(). With
+        # self.shutdown_torrent_pause_list it is possible to have a case when
+        # torrent_id is removed from it in self.on_alert_torrent_paused()
+        # before we call self.save_resume_data() here.
+        save_resume_data_list = []
         for key in self.torrents.keys():
             if not self.torrents[key].handle.is_paused():
                 # We set auto_managed false to prevent lt from resuming the torrent
                 self.torrents[key].handle.auto_managed(False)
                 self.torrents[key].handle.pause()
                 self.shutdown_torrent_pause_list.append(key)
+                save_resume_data_list.append(key)
+
+        self.save_resume_data(save_resume_data_list)
+
         # We have to wait for all torrents to pause and write their resume data
         wait = True
         while wait:
@@ -224,7 +253,11 @@ class TorrentManager(component.Component):
 
     def update(self):
         for torrent_id, torrent in self.torrents.items():
-            if self.config["stop_seed_at_ratio"] or torrent.options["stop_at_ratio"] and torrent.state not in ("Checking", "Allocating"):
+            if self.config["stop_seed_at_ratio"] or torrent.options["stop_at_ratio"] and torrent.state not in ("Checking", "Allocating", "Paused", "Queued"):
+                # If the global setting is set, but the per-torrent isn't.. Just skip to the next torrent
+                # This is so that a user can turn-off the stop at ratio option on a per-torrent basis
+                if self.config["stop_seed_at_ratio"] and not torrent.options["stop_at_ratio"]:
+                    continue
                 stop_ratio = self.config["stop_seed_ratio"]
                 if torrent.options["stop_at_ratio"]:
                     stop_ratio = torrent.options["stop_ratio"]
@@ -257,15 +290,12 @@ class TorrentManager(component.Component):
 
         return torrent_info
 
-    def get_resume_data_from_file(self, torrent_id):
+    def legacy_get_resume_data_from_file(self, torrent_id):
         """Returns an entry with the resume data or None"""
         fastresume = ""
         try:
-            _file = open(
-                os.path.join(
-                    self.config["state_location"],
-                    torrent_id + ".fastresume"),
-                    "rb")
+            _file = open(os.path.join(get_config_dir(), "state",
+                                      torrent_id + ".fastresume"), "rb")
             fastresume = _file.read()
             _file.close()
         except IOError, e:
@@ -273,8 +303,18 @@ class TorrentManager(component.Component):
 
         return str(fastresume)
 
+    def legacy_delete_resume_data(self, torrent_id):
+        """Deletes the .fastresume file"""
+        path = os.path.join(get_config_dir(), "state",
+                            torrent_id + ".fastresume")
+        log.debug("Deleting fastresume file: %s", path)
+        try:
+            os.remove(path)
+        except Exception, e:
+            log.warning("Unable to delete the fastresume file: %s", e)
+
     def add(self, torrent_info=None, state=None, options=None, save_state=True,
-            filedump=None, filename=None, magnet=None):
+            filedump=None, filename=None, magnet=None, resume_data=None):
         """Add a torrent to the manager and returns it's torrent_id"""
 
         if torrent_info is None and state is None and filedump is None and magnet is None:
@@ -289,6 +329,8 @@ class TorrentManager(component.Component):
                 torrent_info = lt.torrent_info(lt.bdecode(filedump))
             except Exception, e:
                 log.error("Unable to decode torrent file!: %s", e)
+                # XXX: Probably should raise an exception here..
+                return
 
         if torrent_info is None and state:
             # We have no torrent_info so we need to add the torrent with information
@@ -305,20 +347,31 @@ class TorrentManager(component.Component):
             options["compact_allocation"] = state.compact
             options["download_location"] = state.save_path
             options["auto_managed"] = state.auto_managed
+            options["stop_at_ratio"] = state.stop_at_ratio
+            options["stop_ratio"] = state.stop_ratio
+            options["remove_at_ratio"] = state.remove_at_ratio
+            options["move_completed"] = state.move_completed
+            options["move_completed_path"] = state.move_completed_path
             options["add_paused"] = state.paused
 
-            if not state.magnet:
-                add_torrent_params["ti"] =\
-                    self.get_torrent_info_from_file(
-                        os.path.join(self.config["state_location"], state.torrent_id + ".torrent"))
-
-                if not add_torrent_params["ti"]:
-                    log.error("Unable to add torrent!")
-                    return
-            else:
+            ti = self.get_torrent_info_from_file(
+                    os.path.join(get_config_dir(),
+                                    "state", state.torrent_id + ".torrent"))
+            if ti:
+                add_torrent_params["ti"] = ti
+            elif state.magnet:
                 magnet = state.magnet
+            else:
+                log.error("Unable to add torrent!")
+                return
 
-            add_torrent_params["resume_data"] = self.get_resume_data_from_file(state.torrent_id)
+            # Handle legacy case with storing resume data in individual files
+            # for each torrent
+            if resume_data is None:
+                resume_data = self.legacy_get_resume_data_from_file(state.torrent_id)
+                self.legacy_delete_resume_data(state.torrent_id)
+
+            add_torrent_params["resume_data"] = resume_data
         else:
             # We have a torrent_info object so we're not loading from state.
             # Check if options is None and load defaults
@@ -329,6 +382,13 @@ class TorrentManager(component.Component):
                 o.update(options)
                 options = o
 
+            # Check for renamed files and if so, rename them in the torrent_info
+            # before adding to the session.
+            if options["mapped_files"]:
+                for index, name in options["mapped_files"].items():
+                    log.debug("renaming file index %s to %s", index, name)
+                    torrent_info.rename_file(index, name.encode("utf-8"))
+
             add_torrent_params["ti"] = torrent_info
             add_torrent_params["resume_data"] = ""
 
@@ -392,7 +452,7 @@ class TorrentManager(component.Component):
         # Write the .torrent file to the state directory
         if filedump:
             try:
-                save_file = open(os.path.join(self.config["state_location"],
+                save_file = open(os.path.join(get_config_dir(), "state",
                         torrent.torrent_id + ".torrent"),
                         "wb")
                 save_file.write(filedump)
@@ -417,7 +477,7 @@ class TorrentManager(component.Component):
             self.save_state()
 
         # Emit the torrent_added signal
-        self.signals.emit("torrent_added", torrent.torrent_id)
+        component.get("EventManager").emit(TorrentAddedEvent(torrent.torrent_id))
 
         return torrent.torrent_id
 
@@ -429,7 +489,7 @@ class TorrentManager(component.Component):
             log.debug("Attempting to open %s for add.", torrent_id)
             _file = open(
                 os.path.join(
-                    self.config["state_location"], torrent_id + ".torrent"),
+                    get_config_dir(), "state", torrent_id + ".torrent"),
                         "rb")
             filedump = lt.bdecode(_file.read())
             _file.close()
@@ -440,7 +500,27 @@ class TorrentManager(component.Component):
         return filedump
 
     def remove(self, torrent_id, remove_data=False):
-        """Remove a torrent from the manager"""
+        """
+        Remove a torrent from the session.
+
+        :param torrent_id: the torrent to remove
+        :type torrent_id: string
+        :param remove_data: if True, remove the downloaded data
+        :type remove_data: bool
+
+        :returns: True if removed successfully, False if not
+        :rtype: bool
+
+        :raises InvalidTorrentError: if the torrent_id is not in the session
+
+        """
+
+        if torrent_id not in self.torrents:
+            raise InvalidTorrentError("torrent_id not in session")
+
+        # Emit the signal to the clients
+        component.get("EventManager").emit(PreTorrentRemovedEvent(torrent_id))
+
         try:
             self.session.remove_torrent(self.torrents[torrent_id].handle,
                 1 if remove_data else 0)
@@ -448,8 +528,10 @@ class TorrentManager(component.Component):
             log.warning("Error removing torrent: %s", e)
             return False
 
-        # Remove the .fastresume if it exists
-        self.torrents[torrent_id].delete_fastresume()
+        # Remove fastresume data if it is exists
+        resume_data = self.load_resume_data_file()
+        resume_data.pop(torrent_id, None)
+        self.save_resume_data_file(resume_data)
 
         # Remove the .torrent file in the state
         self.torrents[torrent_id].delete_torrentfile()
@@ -464,7 +546,7 @@ class TorrentManager(component.Component):
         self.save_state()
 
         # Emit the signal to the clients
-        self.signals.emit("torrent_removed", torrent_id)
+        component.get("EventManager").emit(TorrentRemovedEvent(torrent_id))
 
         return True
 
@@ -475,7 +557,7 @@ class TorrentManager(component.Component):
         try:
             log.debug("Opening torrent state file for load.")
             state_file = open(
-                os.path.join(self.config["state_location"], "torrents.state"), "rb")
+                os.path.join(get_config_dir(), "state", "torrents.state"), "rb")
             state = cPickle.load(state_file)
             state_file.close()
         except (EOFError, IOError, Exception), e:
@@ -483,33 +565,29 @@ class TorrentManager(component.Component):
 
         # Try to use an old state
         try:
-            if dir(state.torrents[0]) != dir(TorrentState()):
-                for attr in (set(dir(TorrentState())) - set(dir(state.torrents[0]))):
+            state_tmp = TorrentState()
+            if dir(state.torrents[0]) != dir(state_tmp):
+                for attr in (set(dir(state_tmp)) - set(dir(state.torrents[0]))):
                     for s in state.torrents:
-                        setattr(s, attr, getattr(TorrentState(), attr, None))
+                        setattr(s, attr, getattr(state_tmp, attr, None))
         except Exception, e:
             log.warning("Unable to update state file to a compatible version: %s", e)
 
         # Reorder the state.torrents list to add torrents in the correct queue
         # order.
-        ordered_state = []
-        for torrent_state in state.torrents:
-            for t in ordered_state:
-                if torrent_state.queue < t.queue:
-                    ordered_state.insert(ordered_state.index(t), torrent_state)
-                    break
-            if torrent_state not in ordered_state:
-                ordered_state.append(torrent_state)
+        state.torrents.sort(key=operator.attrgetter("queue"))
 
-        for torrent_state in ordered_state:
+        resume_data = self.load_resume_data_file()
+
+        for torrent_state in state.torrents:
             try:
-                self.add(state=torrent_state, save_state=False)
+                self.add(state=torrent_state, save_state=False,
+                         resume_data=resume_data.get(torrent_state.torrent_id))
             except AttributeError, e:
-                log.error("Torrent state file is either corrupt or incompatible!")
+                log.error("Torrent state file is either corrupt or incompatible! %s", e)
                 break
 
-        # Run the post_session_load plugin hooks
-        self.plugins.run_post_session_load()
+        component.get("EventManager").emit(SessionStartedEvent())
 
     def save_state(self):
         """Save the state of the TorrentManager to the torrents.state file"""
@@ -540,6 +618,8 @@ class TorrentManager(component.Component):
                 torrent.options["stop_ratio"],
                 torrent.options["stop_at_ratio"],
                 torrent.options["remove_at_ratio"],
+                torrent.options["move_completed"],
+                torrent.options["move_completed_path"],
                 torrent.magnet,
                 torrent.time_added
             )
@@ -548,9 +628,8 @@ class TorrentManager(component.Component):
         # Pickle the TorrentManagerState object
         try:
             log.debug("Saving torrent state file.")
-            state_file = open(
-                os.path.join(self.config["state_location"], "torrents.state.new"),
-                                                                        "wb")
+            state_file = open(os.path.join(get_config_dir(),
+                              "state", "torrents.state.new"), "wb")
             cPickle.dump(state, state_file)
             state_file.flush()
             os.fsync(state_file.fileno())
@@ -562,8 +641,8 @@ class TorrentManager(component.Component):
         # We have to move the 'torrents.state.new' file to 'torrents.state'
         try:
             shutil.move(
-                os.path.join(self.config["state_location"], "torrents.state.new"),
-                os.path.join(self.config["state_location"], "torrents.state"))
+                os.path.join(get_config_dir(), "state", "torrents.state.new"),
+                os.path.join(get_config_dir(), "state", "torrents.state"))
         except IOError:
             log.warning("Unable to save state file.")
             return True
@@ -571,10 +650,71 @@ class TorrentManager(component.Component):
         # We return True so that the timer thread will continue
         return True
 
-    def save_resume_data(self):
-        """Saves resume data for all the torrents"""
-        for torrent in self.torrents.values():
-            torrent.save_resume_data()
+    def save_resume_data(self, torrent_ids=None):
+        """
+        Saves resume data for list of torrent_ids or for all torrents if
+        torrent_ids is None
+        """
+
+        if torrent_ids is None:
+            torrent_ids = self.torrents.keys()
+
+        for torrent_id in torrent_ids:
+            self.torrents[torrent_id].save_resume_data()
+
+        self.num_resume_data = len(torrent_ids)
+
+    def load_resume_data_file(self):
+        resume_data = {}
+        try:
+            log.debug("Opening torrents fastresume file for load.")
+            fastresume_file = open(os.path.join(get_config_dir(), "state",
+                                                "torrents.fastresume"), "rb")
+            resume_data = lt.bdecode(fastresume_file.read())
+            fastresume_file.close()
+        except (EOFError, IOError, Exception), e:
+            log.warning("Unable to load fastresume file: %s", e)
+
+        # If the libtorrent bdecode doesn't happen properly, it will return None
+        # so we need to make sure we return a {}
+        if resume_data is None:
+            return {}
+
+        return resume_data
+
+    def save_resume_data_file(self, resume_data=None):
+        """
+        Saves the resume data file with the contents of self.resume_data.  If
+        `resume_data` is None, then we grab the resume_data from the file on
+        disk, else, we update `resume_data` with self.resume_data and save
+        that to disk.
+
+        :param resume_data: the current resume_data, this will be loaded from disk if not provided
+        :type resume_data: dict
+
+        """
+        # Check to see if we're waiting on more resume data
+        if self.num_resume_data or not self.resume_data:
+            return
+
+        path = os.path.join(get_config_dir(), "state", "torrents.fastresume")
+
+        # First step is to load the existing file and update the dictionary
+        if resume_data is None:
+            resume_data = self.load_resume_data_file()
+
+        resume_data.update(self.resume_data)
+        self.resume_data = {}
+
+        try:
+            log.debug("Saving fastresume file: %s", path)
+            fastresume_file = open(path, "wb")
+            fastresume_file.write(lt.bencode(resume_data))
+            fastresume_file.flush()
+            os.fsync(fastresume_file.fileno())
+            fastresume_file.close()
+        except IOError:
+            log.warning("Error trying to save fastresume file")
 
     def queue_top(self, torrent_id):
         """Queue torrent to top"""
@@ -633,31 +773,38 @@ class TorrentManager(component.Component):
     ## Alert handlers ##
     def on_alert_torrent_finished(self, alert):
         log.debug("on_alert_torrent_finished")
-        # Get the torrent_id
         try:
             torrent = self.torrents[str(alert.handle.info_hash())]
         except:
             return
         torrent_id = str(alert.handle.info_hash())
         log.debug("%s is finished..", torrent_id)
+
+        # Get the total_download and if it's 0, do not move.. It's likely
+        # that the torrent wasn't downloaded, but just added.
+        total_download = torrent.get_status(["total_payload_download"])["total_payload_download"]
+
         # Move completed download to completed folder if needed
-        if not torrent.is_finished:
+        if not torrent.is_finished and total_download:
             move_path = None
+
             if torrent.options["move_completed"]:
                 move_path = torrent.options["move_completed_path"]
-            elif self.config["move_completed"]:
-                move_path = self.config["move_completed_path"]
-            # Get the total_download and if it's 0, do not move.. It's likely
-            # that the torrent wasn't downloaded, but just added.
-            total_download = torrent.get_status(["total_payload_download"])["total_payload_download"]
-            if move_path and total_download:
                 if torrent.options["download_location"] != move_path:
                     torrent.move_storage(move_path)
+
             torrent.is_finished = True
-            component.get("SignalManager").emit("torrent_finished", torrent_id)
+            component.get("EventManager").emit(TorrentFinishedEvent(torrent_id))
 
         torrent.update_state()
-        torrent.save_resume_data()
+
+        # Only save resume data if it was actually downloaded something. Helps
+        # on startup with big queues with lots of seeding torrents. Libtorrent
+        # emits alert_torrent_finished for them, but there seems like nothing
+        # worth really to save in resume data, we just read it up in
+        # self.load_state().
+        if total_download:
+            self.save_resume_data((torrent_id, ))
 
     def on_alert_torrent_paused(self, alert):
         log.debug("on_alert_torrent_paused")
@@ -665,35 +812,39 @@ class TorrentManager(component.Component):
             torrent = self.torrents[str(alert.handle.info_hash())]
         except:
             return
-        # Get the torrent_id
         torrent_id = str(alert.handle.info_hash())
         # Set the torrent state
+        old_state = torrent.state
         torrent.update_state()
-        component.get("SignalManager").emit("torrent_paused", torrent_id)
+        if torrent.state != old_state:
+            component.get("EventManager").emit(TorrentStateChangedEvent(torrent_id, torrent.state))
 
-        # Write the fastresume file
-        torrent.save_resume_data()
+        # Don't save resume data for each torrent after self.stop() was called.
+        # We save resume data in bulk in self.stop() in this case.
+        if self.save_resume_data_timer.running:
+            # Write the fastresume file
+            self.save_resume_data((torrent_id, ))
 
         if torrent_id in self.shutdown_torrent_pause_list:
             self.shutdown_torrent_pause_list.remove(torrent_id)
 
     def on_alert_torrent_checked(self, alert):
         log.debug("on_alert_torrent_checked")
-        # Get the torrent_id
         try:
             torrent = self.torrents[str(alert.handle.info_hash())]
         except:
             return
+
         # Set the torrent state
         torrent.update_state()
 
     def on_alert_tracker_reply(self, alert):
         log.debug("on_alert_tracker_reply: %s", alert.message().decode("utf8"))
-        # Get the torrent_id
         try:
             torrent = self.torrents[str(alert.handle.info_hash())]
         except:
             return
+
         # Set the tracker status for the torrent
         if alert.message() != "Got peers from DHT":
             torrent.set_tracker_status(_("Announce OK"))
@@ -706,7 +857,6 @@ class TorrentManager(component.Component):
 
     def on_alert_tracker_announce(self, alert):
         log.debug("on_alert_tracker_announce")
-        # Get the torrent_id
         try:
             torrent = self.torrents[str(alert.handle.info_hash())]
         except:
@@ -715,22 +865,8 @@ class TorrentManager(component.Component):
         # Set the tracker status for the torrent
         torrent.set_tracker_status(_("Announce Sent"))
 
-    def on_alert_tracker(self, alert):
-        log.debug("on_alert_tracker")
-        # Get the torrent_id
-        try:
-            torrent = self.torrents[str(alert.handle.info_hash())]
-        except:
-            return
-
-        tracker_status = "%s: %s" % \
-            (_("Alert"), str(alert.message()).strip('"')[8:])
-        # Set the tracker status for the torrent
-        torrent.set_tracker_status(tracker_status)
-
     def on_alert_tracker_warning(self, alert):
         log.debug("on_alert_tracker_warning")
-        # Get the torrent_id
         try:
             torrent = self.torrents[str(alert.handle.info_hash())]
         except:
@@ -762,26 +898,46 @@ class TorrentManager(component.Component):
             torrent = self.torrents[str(alert.handle.info_hash())]
         except:
             return
+        torrent_id = str(alert.handle.info_hash())
         torrent.is_finished = torrent.handle.is_seed()
+        old_state = torrent.state
         torrent.update_state()
+        if torrent.state != old_state:
+            # We need to emit a TorrentStateChangedEvent too
+            component.get("EventManager").emit(TorrentStateChangedEvent(torrent_id, torrent.state))
+        component.get("EventManager").emit(TorrentResumedEvent(torrent_id))
 
     def on_alert_state_changed(self, alert):
         log.debug("on_alert_state_changed")
         try:
-            torrent = self.torrents[str(alert.handle.info_hash())]
+            torrent_id = str(alert.handle.info_hash())
+            torrent = self.torrents[torrent_id]
         except:
             return
-        torrent_id = str(alert.handle.info_hash())
+
+        old_state = torrent.state
         torrent.update_state()
-        component.get("SignalManager").emit("torrent_state_changed", torrent_id)
+        # Only emit a state changed event if the state has actually changed
+        if torrent.state != old_state:
+            component.get("EventManager").emit(TorrentStateChangedEvent(torrent_id, torrent.state))
 
     def on_alert_save_resume_data(self, alert):
         log.debug("on_alert_save_resume_data")
+
+        torrent_id = str(alert.handle.info_hash())
+
         try:
-            torrent = self.torrents[str(alert.handle.info_hash())]
+            torrent = self.torrents[torrent_id]
         except:
             return
-        torrent.write_resume_data(alert.resume_data)
+
+        # Libtorrent in add_torrent() expects resume_data to be bencoded
+        self.resume_data[torrent_id] = lt.bencode(alert.resume_data)
+        self.num_resume_data -= 1
+
+        torrent.waiting_on_resume_data = False
+
+        self.save_resume_data_file()
 
     def on_alert_save_resume_data_failed(self, alert):
         log.debug("on_alert_save_resume_data_failed: %s", alert.message())
@@ -789,8 +945,13 @@ class TorrentManager(component.Component):
             torrent = self.torrents[str(alert.handle.info_hash())]
         except:
             return
+
+        self.num_resume_data -= 1
         torrent.waiting_on_resume_data = False
 
+        self.save_resume_data_file()
+
+
     def on_alert_file_renamed(self, alert):
         log.debug("on_alert_file_renamed")
         log.debug("index: %s name: %s", alert.index, alert.name.decode("utf8"))
@@ -800,8 +961,6 @@ class TorrentManager(component.Component):
             return
         torrent_id = str(alert.handle.info_hash())
 
-        torrent.files[alert.index]["path"] = alert.name
-
         # We need to see if this file index is in a waiting_on_folder list
         folder_rename = False
         for i, wait_on_folder in enumerate(torrent.waiting_on_folder_rename):
@@ -809,8 +968,9 @@ class TorrentManager(component.Component):
                 folder_rename = True
                 if len(wait_on_folder[2]) == 1:
                     # This is the last alert we were waiting for, time to send signal
-                    component.get("SignalManager").emit("torrent_folder_renamed", torrent_id, wait_on_folder[0], wait_on_folder[1])
+                    component.get("EventManager").emit(TorrentFolderRenamedEvent(torrent_id, wait_on_folder[0], wait_on_folder[1]))
                     del torrent.waiting_on_folder_rename[i]
+                    self.save_resume_data((torrent_id,))
                     break
                 # This isn't the last file to be renamed in this folder, so just
                 # remove the index and continue
@@ -818,7 +978,8 @@ class TorrentManager(component.Component):
 
         if not folder_rename:
             # This is just a regular file rename so send the signal
-            component.get("SignalManager").emit("torrent_file_renamed", torrent_id, alert.index, alert.name)
+            component.get("EventManager").emit(TorrentFileRenamedEvent(torrent_id, alert.index, alert.name))
+            self.save_resume_data((torrent_id,))
 
     def on_alert_metadata_received(self, alert):
         log.debug("on_alert_metadata_received")
