@@ -31,7 +31,6 @@
 #    this exception statement from your version. If you delete this exception
 #    statement from all source files in the program, then also delete it here.
 #
-
 #
 
 
@@ -40,16 +39,20 @@ pygtk.require('2.0')
 import gtk, gtk.glade
 import gettext
 import gobject
+import base64
+import os
 
 import pkg_resources
 
-from deluge.ui.client import aclient as client
+from deluge.ui.client import client
 import deluge.component as component
-import deluge.ui.gtkui.listview as listview
+import listview
 from deluge.configmanager import ConfigManager
 from deluge.log import LOG as log
 import deluge.common
 import deluge.ui.common
+import dialogs
+import common
 
 class AddTorrentDialog(component.Component):
     def __init__(self):
@@ -85,6 +88,7 @@ class AddTorrentDialog(component.Component):
         self.infos = {}
         self.core_config = {}
         self.options = {}
+
         self.previous_selected_torrent = None
 
 
@@ -105,6 +109,8 @@ class AddTorrentDialog(component.Component):
         column.pack_start(render, False)
         column.add_attribute(render, "stock-id", 5)
         render = gtk.CellRendererText()
+        render.set_property("editable", True)
+        render.connect("edited", self._on_filename_edited)
         column.pack_start(render, True)
         column.add_attribute(render, "text", 1)
         column.set_expand(True)
@@ -142,8 +148,9 @@ class AddTorrentDialog(component.Component):
         self.update_core_config()
 
     def show(self, focus=False):
-        self.update_core_config()
+        return self.update_core_config(True, focus)
 
+    def _show(self, focus=False):
         if client.is_localhost():
             self.glade.get_widget("button_location").show()
             self.glade.get_widget("entry_download_path").hide()
@@ -169,41 +176,44 @@ class AddTorrentDialog(component.Component):
         self.dialog.set_transient_for(component.get("MainWindow").window)
         return None
 
-    def update_core_config(self):
-        self.core_config = {}
-
+    def update_core_config(self, show=False, focus=False):
         def _on_config_values(config):
             self.core_config = config
+            self.set_default_options()
+            if show:
+                self._show(focus)
 
         # Send requests to the core for these config values
-        client.get_config_values(_on_config_values, self.core_keys)
-        # Force a call to the core because we need this data now
-        client.force_call()
-        self.set_default_options()
+        return client.core.get_config_values(self.core_keys).addCallback(_on_config_values)
 
     def add_from_files(self, filenames):
         import os.path
         new_row = None
 
         for filename in filenames:
+            # Convert the path to unicode
+            filename = unicode(filename)
+
             # Get the torrent data from the torrent file
             try:
                 info = deluge.ui.common.TorrentInfo(filename)
             except Exception, e:
                 log.debug("Unable to open torrent file: %s", e)
-                log.exception(e)
+                dialogs.ErrorDialog(_("Invalid File"), e, self.dialog).run()
                 continue
 
             if info.info_hash in self.files:
                 log.debug("Trying to add a duplicate torrent!")
+                dialogs.ErrorDialog(_("Duplicate Torrent"), _("You cannot add the same torrent twice."), self.dialog).run()
                 continue
 
             name = "%s (%s)" % (info.name, os.path.split(filename)[-1])
             new_row = self.torrent_liststore.append(
                 [info.info_hash, info.name, filename])
             self.files[info.info_hash] = info.files
-            self.infos[info.info_hash] = info.metadata
+            self.infos[info.info_hash] = info.filedata
             self.listview_torrents.get_selection().select_iter(new_row)
+
             self.set_default_options()
             self.save_torrent_options(new_row)
 
@@ -216,7 +226,11 @@ class AddTorrentDialog(component.Component):
         new_row = None
 
         for uri in uris:
-            info_hash = base64.b32decode(uri.split("&")[0][20:]).encode("hex")
+            s = uri.split("&")[0][20:]
+            if len(s) == 32:
+                info_hash = base64.b32decode(s).encode("hex")
+            elif len(s) == 40:
+                info_hash = s
             if info_hash in self.infos:
                 log.debug("Torrent already in list!")
                 continue
@@ -282,7 +296,7 @@ class AddTorrentDialog(component.Component):
         self.listview_files.expand_row("0", False)
 
     def prepare_file(self, file, file_name, file_num, download, files_storage):
-        first_slash_index = file_name.find("/")
+        first_slash_index = file_name.find(os.path.sep)
         if first_slash_index == -1:
             files_storage[file_name] = (file_num, file, download)
         else:
@@ -295,7 +309,7 @@ class AddTorrentDialog(component.Component):
     def add_files(self, parent_iter, split_files):
         ret = 0
         for key,value in split_files.iteritems():
-            if key.endswith("/"):
+            if key.endswith(os.path.sep):
                 chunk_iter = self.files_treestore.append(parent_iter,
                                 [True, key, 0, -1, False, gtk.STOCK_DIRECTORY])
                 chunk_size = self.add_files(chunk_iter, value)
@@ -371,7 +385,11 @@ class AddTorrentDialog(component.Component):
 
         torrent_id = self.torrent_liststore.get_value(row, 0)
 
-        options = {}
+        if torrent_id in self.options:
+            options = self.options[torrent_id]
+        else:
+            options = {}
+
         if client.is_localhost():
             options["download_location"] = \
                 self.glade.get_widget("button_location").get_current_folder()
@@ -416,7 +434,7 @@ class AddTorrentDialog(component.Component):
             if self.files_treestore.iter_has_child(iter):
                 self.build_priorities(self.files_treestore.iter_children(iter),
                                           priorities)
-            elif not self.files_treestore.get_value(iter, 1).endswith("/"):
+            elif not self.files_treestore.get_value(iter, 1).endswith(os.path.sep):
                 priorities[self.files_treestore.get_value(iter, 3)] = self.files_treestore.get_value(iter, 0)
             iter = self.files_treestore.iter_next(iter)
         return priorities
@@ -462,6 +480,7 @@ class AddTorrentDialog(component.Component):
         # Check to see if we can change file priorities
         (model, row) = self.listview_torrents.get_selection().get_selected()
         if self.options[model[row][0]]["compact_allocation"]:
+            dialogs.InformationDialog(_("Unable to set file priority!"), _("File prioritization is unavailable when using Compact allocation."), self.dialog).run()
             return
         (model, paths) = self.listview_files.get_selection().get_selected_rows()
         if len(paths) > 1:
@@ -577,29 +596,62 @@ class AddTorrentDialog(component.Component):
         else:
             url = None
 
+        entry.set_text("")
+        dialog.hide()
+
         # This is where we need to fetch the .torrent file from the URL and
         # add it to the list.
         log.debug("url: %s", url)
-        if url != None:
+        if url:
             if deluge.common.is_url(url):
                 self.add_from_url(url)
             elif deluge.common.is_magnet(url):
                 self.add_from_magnets([url])
-
-        entry.set_text("")
-        dialog.hide()
+            else:
+                dialogs.ErrorDialog(_("Invalid URL"), _("%s is not a valid URL." % url), self.dialog).run()
 
     def add_from_url(self, url):
-        gobject.idle_add(self._download_from_url, url)
+        dialog = gtk.Dialog(
+            _("Downloading.."),
+            flags=gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT | gtk.DIALOG_NO_SEPARATOR,
+            parent=self.dialog)
+        dialog.set_transient_for(self.dialog)
 
-    def _download_from_url(self, url):
-        import urllib
+        pb = gtk.ProgressBar()
+        dialog.vbox.pack_start(pb, True, True)
+        dialog.show_all()
+
+        # Create a tmp file path
         import tempfile
         import os.path
         tmp_file = os.path.join(tempfile.gettempdir(), url.split("/")[-1])
-        filename, headers = urllib.urlretrieve(url, tmp_file)
-        log.debug("filename: %s", filename)
-        self.add_from_files([filename])
+
+        def on_part(data, current_length, total_length):
+            if total_length:
+                percent = float(current_length) / float(total_length)
+                pb.set_fraction(percent)
+                pb.set_text("%.2f%% (%s / %s)" % (
+                    percent * 100,
+                    deluge.common.fsize(current_length),
+                    deluge.common.fsize(total_length)))
+            else:
+                pb.pulse()
+                pb.set_text("%s" % deluge.common.fsize(current_length))
+
+        def on_download_success(result):
+            log.debug("Download success!")
+            self.add_from_files([result])
+            dialog.destroy()
+
+        def on_download_fail(result):
+            log.debug("Download failed: %s", result)
+            dialog.destroy()
+            dialogs.ErrorDialog(_("Download Failed"), _("Failed to download : %s" % url), details=result.getErrorMessage(), parent=self.dialog).run()
+
+        import deluge.httpdownloader
+        d = deluge.httpdownloader.download_file(url, tmp_file, on_part)
+        d.addCallback(on_download_success)
+        d.addErrback(on_download_fail)
 
     def _on_button_hash_clicked(self, widget):
         log.debug("_on_button_hash_clicked")
@@ -659,11 +711,6 @@ class AddTorrentDialog(component.Component):
         if row is not None:
             self.save_torrent_options(row)
 
-        torrent_filenames = []
-        torrent_magnets = []
-        torrent_magnet_options = []
-        torrent_options = []
-
         row = self.torrent_liststore.get_iter_first()
         while row != None:
             torrent_id = self.torrent_liststore.get_value(row, 0)
@@ -678,21 +725,16 @@ class AddTorrentDialog(component.Component):
                 options["file_priorities"] = file_priorities
 
             if deluge.common.is_magnet(filename):
-                torrent_magnets.append(filename)
                 del options["file_priorities"]
-                torrent_magnet_options.append(options)
+                client.core.add_torrent_magnet(filename, options)
             else:
-                torrent_filenames.append(filename)
-                torrent_options.append(options)
+                client.core.add_torrent_file(
+                    os.path.split(filename)[-1],
+                    base64.encodestring(self.infos[torrent_id]),
+                    options)
 
             row = self.torrent_liststore.iter_next(row)
 
-        if torrent_filenames:
-            client.add_torrent_file(torrent_filenames, torrent_options)
-        if torrent_magnets:
-            client.add_torrent_magnets(torrent_magnets, torrent_magnet_options)
-
-        client.force_call(False)
         self.hide()
 
     def _on_button_apply_clicked(self, widget):
@@ -725,3 +767,118 @@ class AddTorrentDialog(component.Component):
     def _on_delete_event(self, widget, event):
         self.hide()
         return True
+
+    def get_file_path(self, row, path=""):
+        if not row:
+            return path
+
+        path = self.files_treestore[row][1] + path
+        return self.get_file_path(self.files_treestore.iter_parent(row), path)
+
+    def _on_filename_edited(self, renderer, path, new_text):
+        index = self.files_treestore[path][3]
+
+        # Return if the text hasn't changed
+        if new_text == self.files_treestore[path][1]:
+            return
+
+        # Get the tree iter
+        itr = self.files_treestore.get_iter(path)
+
+        # Get the torrent_id
+        (model, row) = self.listview_torrents.get_selection().get_selected()
+        torrent_id = model[row][0]
+
+        if "mapped_files" not in self.options[torrent_id]:
+            self.options[torrent_id]["mapped_files"] = {}
+
+        if index > -1:
+            # We're renaming a file! Yay! That's easy!
+            parent = self.files_treestore.iter_parent(itr)
+            file_path = self.get_file_path(parent)
+            file_path += new_text
+            file_path = file_path.replace(os.path.sep*2, os.path.sep)
+
+            if "/" in new_text:
+                # There are folders in this path, so we need to create them
+                # and then move the file iter to top
+                split_text = new_text.split(os.path.sep)
+                for s in split_text[:-1]:
+                    parent = self.files_treestore.append(parent,
+                                [True, s, 0, -1, False, gtk.STOCK_DIRECTORY])
+
+                self.files_treestore[itr][1] = split_text[-1]
+                common.reparent_iter(self.files_treestore, itr, parent)
+            else:
+                # Update the row's text
+                self.files_treestore[itr][1] = new_text
+
+            # Update the mapped_files dict in the options with the index and new
+            # file path.
+            # We'll send this to the core when adding the torrent so it knows
+            # what to rename before adding.
+            self.options[torrent_id]["mapped_files"][index] = file_path
+        else:
+            # Folder!
+            def walk_tree(row):
+                if not row:
+                    return
+
+                # Get the file path base once, since it will be the same for
+                # all siblings
+                file_path_base = self.get_file_path(self.files_treestore.iter_parent(row))
+
+                # Iterate through all the siblings at this level
+                while row:
+                    # We recurse if there are children
+                    if self.files_treestore.iter_has_child(row):
+                        walk_tree(self.files_treestore.iter_children(row))
+
+                    index = self.files_treestore[row][3]
+
+                    # Don't do anything if this is a folder
+                    if index == -1:
+                        return
+
+                    # Get the new full path for this file
+                    file_path = file_path_base + self.files_treestore[row][1]
+
+                    # Update the file path in the mapped_files dict
+                    self.options[torrent_id]["mapped_files"][index] = file_path
+
+                    # Get the next siblings iter
+                    row = self.files_treestore.iter_next(row)
+
+            # Update the treestore row first so that when walking the tree
+            # we can construct the new proper paths
+            if len(new_text) == 0 or new_text[-1] != os.path.sep:
+                new_text += os.path.sep
+
+            # We need to check if this folder has been split
+            split_text = new_text[:-1].split(os.path.sep)
+            if len(split_text) > 1:
+                # It's been split, so we need to add new folders and then reparent
+                # itr.
+                parent = self.files_treestore.iter_parent(itr)
+                for s in split_text[:-1]:
+                    # We don't iterate over the last item because we'll just use
+                    # the existing itr and change the text
+                    parent = self.files_treestore.append(parent,
+                                [True, s + os.path.sep, 0, -1, False, gtk.STOCK_DIRECTORY])
+
+                self.files_treestore[itr][1] = split_text[-1] + os.path.sep
+
+                # Now reparent itr to parent
+                common.reparent_iter(self.files_treestore, itr, parent)
+
+                # We need to re-expand the view because it might contracted
+                # if we change the root iter
+                self.listview_files.expand_row("0", False)
+            else:
+                # This was a simple folder rename without any splits, so just
+                # change the path for itr
+                self.files_treestore[itr][1] = new_text
+
+            # Walk through the tree from 'itr' and add all the new file paths
+            # to the 'mapped_files' option
+            walk_tree(itr)
