@@ -31,19 +31,22 @@
 #    this exception statement from your version. If you delete this exception
 #    statement from all source files in the program, then also delete it here.
 #
+#
 
-import gtk, gtk.glade
+
+import gtk, gtk.glade, gtk.gdk
 import gobject
 import gettext
 import os.path
 import cPickle
 
 from deluge.ui.gtkui.torrentdetails import Tab
-from deluge.ui.client import aclient as client
+from deluge.ui.client import client
 from deluge.configmanager import ConfigManager
 import deluge.configmanager
 import deluge.component as component
 import deluge.common
+import common
 
 from deluge.log import LOG as log
 
@@ -178,7 +181,13 @@ class FilesTab(Tab):
             glade.get_widget("menuitem_priority_sep")
         ]
 
+        self.localhost_widgets = [
+            glade.get_widget("menuitem_open_file"),
+            glade.get_widget("menuitem3")
+        ]
+
         self.listview.connect("row-activated", self._on_row_activated)
+        self.listview.connect("key-press-event", self._on_key_press_event)
         self.listview.connect("button-press-event", self._on_button_press_event)
 
         self.listview.enable_model_drag_source(
@@ -199,10 +208,10 @@ class FilesTab(Tab):
             "on_menuitem_expand_all_activate": self._on_menuitem_expand_all_activate
         })
 
-        # Connect to the 'torrent_file_renamed' signal
-        component.get("Signals").connect_to_signal("torrent_file_renamed", self._on_torrent_file_renamed_signal)
-        component.get("Signals").connect_to_signal("torrent_folder_renamed", self._on_torrent_folder_renamed_signal)
-        component.get("Signals").connect_to_signal("torrent_removed", self._on_torrent_removed_signal)
+        # Connect to various events from the daemon
+        client.register_event_handler("TorrentFileRenamedEvent", self._on_torrentfilerenamed_event)
+        client.register_event_handler("TorrentFolderRenamedEvent", self._on_torrentfolderrenamed_event)
+        client.register_event_handler("TorrentRemovedEvent", self._on_torrentremoved_event)
 
         # Attempt to load state
         self.load_state()
@@ -212,6 +221,11 @@ class FilesTab(Tab):
 
         self.torrent_id = None
 
+    def start(self):
+        attr = "hide" if not client.is_localhost() else "show"
+        for widget in self.localhost_widgets:
+            getattr(widget, attr)()
+
     def save_state(self):
         filename = "files_tab.state"
         # Get the current sort order of the view
@@ -296,16 +310,16 @@ class FilesTab(Tab):
             self.torrent_id = torrent_id
             status_keys += ["compact"]
 
-            if self.torrent_id not in self.files_list.keys():
-                # We need to get the files list
-                log.debug("Getting file list from core..")
-                status_keys += ["files"]
-            else:
+            if self.torrent_id in self.files_list:
                 # We already have the files list stored, so just update the view
                 self.update_files()
 
-        client.get_torrent_status(self._on_get_torrent_status, self.torrent_id, status_keys)
-        client.force_call(True)
+        if self.torrent_id not in self.files_list or not self.files_list[self.torrent_id]:
+            # We need to get the files list
+            log.debug("Getting file list from core..")
+            status_keys += ["files"]
+
+        client.core.get_torrent_status(self.torrent_id, status_keys).addCallback(self._on_get_torrent_status)
 
     def clear(self):
         self.treestore.clear()
@@ -313,8 +327,7 @@ class FilesTab(Tab):
 
     def _on_row_activated(self, tree, path, view_column):
         if client.is_localhost:
-            client.get_torrent_status(self._on_open_file, self.torrent_id, ["save_path", "files"])
-            client.force_call(False)
+            client.core.get_torrent_status(self.torrent_id, ["save_path", "files"]).addCallback(self._on_open_file)
 
     def get_file_path(self, row, path=""):
         if not row:
@@ -459,6 +472,17 @@ class FilesTab(Tab):
             self.file_menu.popup(None, None, None, event.button, event.time)
             return True
 
+    def _on_key_press_event(self, widget, event):
+        # Menu key
+        if gtk.gdk.keyval_name(event.keyval) != "Menu":
+            return
+
+        if not self.get_selected_files():
+            return
+
+        self.file_menu.popup(None, None, None, 3, event.time)
+        return True
+
     def _on_menuitem_open_file_activate(self, menuitem):
         self._on_row_activated(None, None, None)
 
@@ -478,7 +502,7 @@ class FilesTab(Tab):
         priorities = [p[1] for p in file_priorities]
         log.debug("priorities: %s", priorities)
 
-        client.set_torrent_file_priorities(self.torrent_id, priorities)
+        client.core.set_torrent_file_priorities(self.torrent_id, priorities)
 
     def _on_menuitem_donotdownload_activate(self, menuitem):
         self._set_file_priorities_on_user_change(
@@ -532,7 +556,7 @@ class FilesTab(Tab):
 
             log.debug("filepath: %s", filepath)
 
-            client.rename_files(self.torrent_id, [(index, filepath)])
+            client.core.rename_files(self.torrent_id, [(index, filepath)])
         else:
             # We are renaming a folder
             folder = self.treestore[path][0]
@@ -543,7 +567,7 @@ class FilesTab(Tab):
                 parent_path = self.treestore[itr][0] + parent_path
                 itr = self.treestore.iter_parent(itr)
 
-            client.rename_folder(self.torrent_id, parent_path + folder, parent_path + new_text)
+            client.core.rename_folder(self.torrent_id, parent_path + folder, parent_path + new_text)
 
         self._editing_index = None
 
@@ -553,8 +577,12 @@ class FilesTab(Tab):
     def _on_filename_editing_canceled(self, renderer):
         self._editing_index = None
 
-    def _on_torrent_file_renamed_signal(self, torrent_id, index, name):
+    def _on_torrentfilerenamed_event(self, torrent_id, index, name):
         log.debug("index: %s name: %s", index, name)
+
+        if torrent_id not in self.files_list:
+            return
+
         old_name = self.files_list[torrent_id][index]["path"]
         self.files_list[torrent_id][index]["path"] = name
 
@@ -667,29 +695,6 @@ class FilesTab(Tab):
 
         return path_iter
 
-    def reparent_iter(self, itr, parent, move_siblings=False):
-        """
-        This effectively moves itr plus it's children to be a child of parent
-
-        If move_siblings is True, it will move all itr's siblings to parent
-        """
-        src = itr
-        def move_children(i, dest):
-            while i:
-                n = self.treestore.append(dest, self.treestore.get(i, *xrange(self.treestore.get_n_columns())))
-                to_remove = i
-                if self.treestore.iter_children(i):
-                    move_children(self.treestore.iter_children(i), n)
-                if i != src:
-                    i = self.treestore.iter_next(i)
-                else:
-                    # This is the source iter, we don't want other iters in it's level
-                    if not move_siblings:
-                        i = None
-                self.treestore.remove(to_remove)
-
-        move_children(itr, parent)
-
     def remove_childless_folders(self, itr):
         """
         Goes up the tree removing childless itrs starting at itr
@@ -699,10 +704,13 @@ class FilesTab(Tab):
             self.treestore.remove(itr)
             itr = parent
 
-    def _on_torrent_folder_renamed_signal(self, torrent_id, old_folder, new_folder):
+    def _on_torrentfolderrenamed_event(self, torrent_id, old_folder, new_folder):
         log.debug("on_torrent_folder_renamed_signal")
         log.debug("old_folder: %s new_folder: %s", old_folder, new_folder)
 
+        if torrent_id not in self.files_list:
+            return
+
         if old_folder[-1] != "/":
             old_folder += "/"
         if new_folder[-1] != "/":
@@ -732,24 +740,24 @@ class FilesTab(Tab):
             new_folder_iter = self.get_iter_at_path(new_folder)
             if len(new_split) == len(old_split):
                 # These are at the same tree depth, so it's a simple rename
-                self.treestore[old_folder_iter][0] = new_folder
+                self.treestore[old_folder_iter][0] = new_split[-1] + "/"
                 return
             if new_folder_iter:
                 # This means that a folder by this name already exists
-                self.reparent_iter(self.treestore.iter_children(old_folder_iter), new_folder_iter)
+                common.reparent_iter(self.treestore, self.treestore.iter_children(old_folder_iter), new_folder_iter)
             else:
                 parent = old_folder_iter_parent
                 for ns in new_split[:-1]:
                     parent = self.treestore.append(parent, [ns + "/", 0, "", 0, 0, -1, gtk.STOCK_DIRECTORY])
 
                 self.treestore[old_folder_iter][0] = new_split[-1] + "/"
-                self.reparent_iter(old_folder_iter, parent)
+                common.reparent_iter(self.treestore, old_folder_iter, parent)
 
             # We need to check if the old_folder_iter_parent no longer has children
             # and if so, we delete it
             self.remove_childless_folders(old_folder_iter_parent)
 
-    def _on_torrent_removed_signal(self, torrent_id):
+    def _on_torrentremoved_event(self, torrent_id):
         if torrent_id in self.files_list:
             del self.files_list[torrent_id]
 
@@ -782,11 +790,11 @@ class FilesTab(Tab):
                 while itr:
                     pp = self.treestore[itr][0] + pp
                     itr = self.treestore.iter_parent(itr)
-                client.rename_folder(self.torrent_id, pp + model[selected[0]][0], parent_path + model[selected[0]][0])
+                client.core.rename_folder(self.torrent_id, pp + model[selected[0]][0], parent_path + model[selected[0]][0])
             else:
                 #[(index, filepath), ...]
                 to_rename = []
                 for s in selected:
                     to_rename.append((model[s][5], parent_path + model[s][0]))
                 log.debug("to_rename: %s", to_rename)
-                client.rename_files(self.torrent_id, to_rename)
+                client.core.rename_files(self.torrent_id, to_rename)
