@@ -31,7 +31,6 @@
 #    this exception statement from your version. If you delete this exception
 #    statement from all source files in the program, then also delete it here.
 #
-
 #
 
 
@@ -43,19 +42,19 @@ import pkg_resources
 from urlparse import urlparse
 import urllib
 
-from deluge.ui.client import aclient as client
+from deluge.ui.client import client
 import deluge.component as component
 from deluge.configmanager import ConfigManager
 from deluge.ui.gtkui.ipcinterface import process_args
 
 import deluge.common
-import deluge.ui.gtkui.common as common
+import common
 
 from deluge.log import LOG as log
 
 class MainWindow(component.Component):
     def __init__(self):
-        component.Component.__init__(self, "MainWindow", interval=2000)
+        component.Component.__init__(self, "MainWindow", interval=2)
         self.config = ConfigManager("gtkui.conf")
         # Get the glade file for the main window
         self.main_glade = gtk.glade.XML(
@@ -63,8 +62,11 @@ class MainWindow(component.Component):
                                                     "glade/main_window.glade"))
 
         self.window = self.main_glade.get_widget("main_window")
-        self.window.set_icon(common.get_logo(32))
+
+        self.window.set_icon(common.get_deluge_icon())
+
         self.vpaned = self.main_glade.get_widget("vpaned")
+        self.initial_vpaned_position = self.config["window_pane_position"]
 
         # Load the window state
         self.load_window_state()
@@ -86,11 +88,18 @@ class MainWindow(component.Component):
 
         self.config.register_set_function("show_rate_in_title", self._on_set_show_rate_in_title, apply_now=False)
 
+        client.register_event_handler("NewVersionAvailableEvent", self.on_newversionavailable_event)
+        client.register_event_handler("TorrentFinishedEvent", self.on_torrentfinished_event)
+
+    def first_show(self):
         if not(self.config["start_in_tray"] and \
                self.config["enable_system_tray"]) and not \
                 self.window.get_property("visible"):
             log.debug("Showing window")
             self.show()
+            while gtk.events_pending():
+                gtk.main_iteration(False)
+            self.vpaned.set_position(self.initial_vpaned_position)
 
     def show(self):
         try:
@@ -102,8 +111,10 @@ class MainWindow(component.Component):
 
         self.window.show()
 
+
     def hide(self):
         component.pause("TorrentView")
+        component.get("TorrentView").save_state()
         component.pause("StatusBar")
         component.pause("TorrentDetails")
         # Store the x, y positions for when we restore the window
@@ -141,7 +152,6 @@ class MainWindow(component.Component):
         return self.main_glade
 
     def quit(self):
-        del self.config
         gtk.main_quit()
 
     def load_window_state(self):
@@ -153,8 +163,6 @@ class MainWindow(component.Component):
         self.window.resize(w, h)
         if self.config["window_maximized"]:
             self.window.maximize()
-        self.vpaned.set_position(
-            self.config["window_height"] - self.config["window_pane_position"])
 
     def on_window_configure_event(self, widget, event):
         if not self.config["window_maximized"] and self.visible:
@@ -195,7 +203,7 @@ class MainWindow(component.Component):
         return True
 
     def on_vpaned_position_event(self, obj, param):
-        self.config["window_pane_position"] = self.config["window_height"] - self.vpaned.get_position()
+        self.config["window_pane_position"] = self.vpaned.get_position()
 
     def on_drag_data_received_event(self, widget, drag_context, x, y, selection_data, info, timestamp):
         args = []
@@ -220,10 +228,20 @@ class MainWindow(component.Component):
             upload_rate = deluge.common.fspeed(status["upload_rate"])
             self.window.set_title("Deluge - %s %s %s %s" % (_("Down:"), download_rate, _("Up:"), upload_rate))
         if self.config["show_rate_in_title"]:
-            client.get_session_status(_on_get_session_status, ["download_rate", "upload_rate"])
+            client.core.get_session_status(["download_rate", "upload_rate"]).addCallback(_on_get_session_status)
 
     def _on_set_show_rate_in_title(self, key, value):
         if value:
             self.update()
         else:
             self.window.set_title("Deluge")
+
+    def on_newversionavailable_event(self, new_version):
+        if self.config["show_new_releases"]:
+            from twisted.internet import reactor
+            from deluge.ui.gtkui.new_release_dialog import NewReleaseDialog
+            reactor.callLater(5.0, NewReleaseDialog().show, new_version)
+
+    def on_torrentfinished_event(self, torrent_id):
+        from deluge.ui.gtkui.notification import Notification
+        Notification().notify(torrent_id)
