@@ -31,7 +31,6 @@
 #    this exception statement from your version. If you delete this exception
 #    statement from all source files in the program, then also delete it here.
 #
-
 #
 
 
@@ -42,9 +41,9 @@ import pkg_resources
 
 import deluge.error
 import deluge.component as component
-from deluge.ui.client import aclient as client
+from deluge.ui.client import client
 import deluge.common
-import deluge.ui.gtkui.common as common
+import common
 from deluge.configmanager import ConfigManager
 
 from deluge.log import LOG as log
@@ -170,11 +169,12 @@ class MenuBar(component.Component):
             "menuitem_addtorrent"
         ]
 
-        if self.config["classic_mode"]:
-            # We need to remove the 'quit and shutdown daemon' menu item
-            self.window.main_glade.get_widget("menuitem_quitdaemon").hide()
-            self.window.main_glade.get_widget("separatormenuitem").hide()
-            self.window.main_glade.get_widget("menuitem_connectionmanager").hide()
+        self.config.register_set_function("classic_mode", self._on_classic_mode)
+
+        client.register_event_handler("TorrentStateChangedEvent", self.on_torrentstatechanged_event)
+        client.register_event_handler("TorrentResumedEvent", self.on_torrentresumed_event)
+        client.register_event_handler("SessionPausedEvent", self.on_sessionpaused_event)
+        client.register_event_handler("SessionResumedEvent", self.on_sessionresumed_event)
 
     def start(self):
         for widget in self.change_sensitivity:
@@ -194,12 +194,11 @@ class MenuBar(component.Component):
             for widget in non_remote_items:
                 self.torrentmenu_glade.get_widget(widget).set_no_show_all(False)
 
-        # Show the Torrent menu because we're connected to a host
-        self.menu_torrent.show()
-
         if not self.config["classic_mode"]:
             self.window.main_glade.get_widget("separatormenuitem").show()
             self.window.main_glade.get_widget("menuitem_quitdaemon").show()
+        # Show the Torrent menu because we're connected to a host
+        self.menu_torrent.show()
 
     def stop(self):
         for widget in self.change_sensitivity:
@@ -229,6 +228,18 @@ class MenuBar(component.Component):
         return sep
 
     ### Callbacks ###
+    def on_torrentstatechanged_event(self, torrent_id, state):
+        if state == "Paused":
+            self.update_menu()
+
+    def on_torrentresumed_event(self, torrent_id):
+        self.update_menu()
+
+    def on_sessionpaused_event(self):
+        self.update_menu()
+
+    def on_sessionresumed_event(self):
+        self.update_menu()
 
     ## File Menu ##
     def on_menuitem_addtorrent_activate(self, data=None):
@@ -243,16 +254,14 @@ class MenuBar(component.Component):
     def on_menuitem_quitdaemon_activate(self, data=None):
         log.debug("on_menuitem_quitdaemon_activate")
         # Tell the core to shutdown
-        client.shutdown()
-        self.window.quit()
+        def on_shutdown(result):
+            self.window.quit()
+        client.daemon.shutdown().addCallback(on_shutdown)
 
     def on_menuitem_quit_activate(self, data=None):
         log.debug("on_menuitem_quit_activate")
-        if self.config["classic_mode"]:
-            try:
-                client.shutdown()
-            except deluge.error.NoCoreError:
-                pass
+        if self.config["classic_mode"] and client.is_classicmode():
+            client.daemon.shutdown()
         self.window.quit()
 
     ## Edit Menu ##
@@ -267,17 +276,17 @@ class MenuBar(component.Component):
     ## Torrent Menu ##
     def on_menuitem_pause_activate(self, data=None):
         log.debug("on_menuitem_pause_activate")
-        client.pause_torrent(
+        client.core.pause_torrent(
             component.get("TorrentView").get_selected_torrents())
 
     def on_menuitem_resume_activate(self, data=None):
         log.debug("on_menuitem_resume_activate")
-        client.resume_torrent(
+        client.core.resume_torrent(
             component.get("TorrentView").get_selected_torrents())
 
     def on_menuitem_updatetracker_activate(self, data=None):
         log.debug("on_menuitem_updatetracker_activate")
-        client.force_reannounce(
+        client.core.force_reannounce(
             component.get("TorrentView").get_selected_torrents())
 
     def on_menuitem_edittrackers_activate(self, data=None):
@@ -290,12 +299,14 @@ class MenuBar(component.Component):
 
     def on_menuitem_remove_activate(self, data=None):
         log.debug("on_menuitem_remove_activate")
-        from removetorrentdialog import RemoveTorrentDialog
-        RemoveTorrentDialog(component.get("TorrentView").get_selected_torrents()).run()
+        torrent_ids = component.get("TorrentView").get_selected_torrents()
+        if torrent_ids:
+            from removetorrentdialog import RemoveTorrentDialog
+            RemoveTorrentDialog(torrent_ids).run()
 
     def on_menuitem_recheck_activate(self, data=None):
         log.debug("on_menuitem_recheck_activate")
-        client.force_recheck(
+        client.core.force_recheck(
             component.get("TorrentView").get_selected_torrents())
 
     def on_menuitem_open_folder_activate(self, data=None):
@@ -303,7 +314,7 @@ class MenuBar(component.Component):
         def _on_torrent_status(status):
             deluge.common.open_file(status["save_path"])
         for torrent_id in component.get("TorrentView").get_selected_torrents():
-            client.get_torrent_status(_on_torrent_status, torrent_id, ["save_path"])
+            client.core.get_torrent_status(torrent_id, ["save_path"]).addCallback(_on_torrent_status)
 
     def on_menuitem_move_activate(self, data=None):
         log.debug("on_menuitem_move_activate")
@@ -316,17 +327,17 @@ class MenuBar(component.Component):
                 gtk.RESPONSE_CANCEL, gtk.STOCK_OK, gtk.RESPONSE_OK))
             chooser.set_local_only(True)
             if not deluge.common.windows_check():
-                chooser.set_icon(common.get_logo(18))
+                chooser.set_icon(common.get_deluge_icon())
                 chooser.set_property("skip-taskbar-hint", True)
             chooser.set_current_folder(config["choose_directory_dialog_path"])
             if chooser.run() == gtk.RESPONSE_OK:
                 result = chooser.get_filename()
                 config["choose_directory_dialog_path"] = result
-                client.move_storage(
+                client.core.move_storage(
                     component.get("TorrentView").get_selected_torrents(), result)
             chooser.destroy()
         else:
-            client.get_torrent_status(self.show_move_storage_dialog, component.get("TorrentView").get_selected_torrent(), ["save_path"])
+            client.core.get_torrent_status(component.get("TorrentView").get_selected_torrent(), ["save_path"]).addCallback(self.show_move_storage_dialog)
             client.force_call(False)
 
     def show_move_storage_dialog(self, status):
@@ -342,26 +353,26 @@ class MenuBar(component.Component):
             if response_id == gtk.RESPONSE_OK:
                 log.debug("Moving torrents to %s", entry.get_text())
                 path = entry.get_text()
-                client.move_storage(component.get("TorrentView").get_selected_torrents(), path)
+                client.core.move_storage(component.get("TorrentView").get_selected_torrents(), path)
             dialog.hide()
         dialog.connect("response", _on_response_event)
         dialog.show()
 
     def on_menuitem_queue_top_activate(self, value):
         log.debug("on_menuitem_queue_top_activate")
-        client.queue_top(None, component.get("TorrentView").get_selected_torrents())
+        client.core.queue_top(component.get("TorrentView").get_selected_torrents())
 
     def on_menuitem_queue_up_activate(self, value):
         log.debug("on_menuitem_queue_up_activate")
-        client.queue_up(None, component.get("TorrentView").get_selected_torrents())
+        client.core.queue_up(component.get("TorrentView").get_selected_torrents())
 
     def on_menuitem_queue_down_activate(self, value):
         log.debug("on_menuitem_queue_down_activate")
-        client.queue_down(None, component.get("TorrentView").get_selected_torrents())
+        client.core.queue_down(component.get("TorrentView").get_selected_torrents())
 
     def on_menuitem_queue_bottom_activate(self, value):
         log.debug("on_menuitem_queue_bottom_activate")
-        client.queue_bottom(None, component.get("TorrentView").get_selected_torrents())
+        client.core.queue_bottom(component.get("TorrentView").get_selected_torrents())
 
     ## View Menu ##
     def on_menuitem_toolbar_toggled(self, value):
@@ -397,10 +408,10 @@ class MenuBar(component.Component):
     def on_menuitem_set_unlimited(self, widget):
         log.debug("widget.name: %s", widget.name)
         funcs = {
-            "menuitem_down_speed": client.set_torrent_max_download_speed,
-            "menuitem_up_speed": client.set_torrent_max_upload_speed,
-            "menuitem_max_connections": client.set_torrent_max_connections,
-            "menuitem_upload_slots": client.set_torrent_max_upload_slots
+            "menuitem_down_speed": client.core.set_torrent_max_download_speed,
+            "menuitem_up_speed": client.core.set_torrent_max_upload_speed,
+            "menuitem_max_connections": client.core.set_torrent_max_connections,
+            "menuitem_upload_slots": client.core.set_torrent_max_upload_slots
         }
         if widget.name in funcs.keys():
             for torrent in component.get("TorrentView").get_selected_torrents():
@@ -409,10 +420,10 @@ class MenuBar(component.Component):
     def on_menuitem_set_other(self, widget):
         log.debug("widget.name: %s", widget.name)
         funcs = {
-            "menuitem_down_speed": client.set_torrent_max_download_speed,
-            "menuitem_up_speed": client.set_torrent_max_upload_speed,
-            "menuitem_max_connections": client.set_torrent_max_connections,
-            "menuitem_upload_slots": client.set_torrent_max_upload_slots
+            "menuitem_down_speed": client.core.set_torrent_max_download_speed,
+            "menuitem_up_speed": client.core.set_torrent_max_upload_speed,
+            "menuitem_max_connections": client.core.set_torrent_max_connections,
+            "menuitem_upload_slots": client.core.set_torrent_max_upload_slots
         }
         # widget: (header, type_str, image_stockid, image_filename, default)
         other_dialog_info = {
@@ -430,11 +441,11 @@ class MenuBar(component.Component):
 
     def on_menuitem_set_automanaged_on(self, widget):
         for torrent in component.get("TorrentView").get_selected_torrents():
-            client.set_torrent_auto_managed(torrent, True)
+            client.core.set_torrent_auto_managed(torrent, True)
 
     def on_menuitem_set_automanaged_off(self, widget):
         for torrent in component.get("TorrentView").get_selected_torrents():
-            client.set_torrent_auto_managed(torrent, False)
+            client.core.set_torrent_auto_managed(torrent, False)
 
     def on_menuitem_sidebar_zero_toggled(self, widget):
         self.config["sidebar_show_zero"] = widget.get_active()
@@ -443,3 +454,19 @@ class MenuBar(component.Component):
     def on_menuitem_sidebar_trackers_toggled(self, widget):
         self.config["sidebar_show_trackers"] = widget.get_active()
         component.get("FilterTreeView").update()
+
+    def _on_classic_mode(self, key, value):
+        items = [
+            "menuitem_quitdaemon",
+            "separatormenuitem",
+            "menuitem_connectionmanager"
+        ]
+        
+        if value:
+            attr = "hide"
+        else:
+            attr = "show"
+        
+        for item in items:
+            getattr(self.window.main_glade.get_widget(item), attr)()
+
