@@ -31,7 +31,6 @@
 #    this exception statement from your version. If you delete this exception
 #    statement from all source files in the program, then also delete it here.
 #
-
 #
 
 
@@ -42,10 +41,10 @@ import pkg_resources
 
 import deluge.component as component
 from deluge.log import LOG as log
-from deluge.ui.client import aclient as client
+from deluge.ui.client import client
 import deluge.common
 import deluge.error
-import deluge.ui.gtkui.common as common
+import common
 from deluge.configmanager import ConfigManager
 import deluge.configmanager
 
@@ -57,10 +56,14 @@ class Preferences(component.Component):
                     pkg_resources.resource_filename("deluge.ui.gtkui",
                                             "glade/preferences_dialog.glade"))
         self.pref_dialog = self.glade.get_widget("pref_dialog")
-        self.pref_dialog.set_icon(common.get_logo(32))
+        self.pref_dialog.set_icon(common.get_deluge_icon())
         self.treeview = self.glade.get_widget("treeview")
         self.notebook = self.glade.get_widget("notebook")
         self.gtkui_config = ConfigManager("gtkui.conf")
+
+        self.glade.get_widget("image_magnet").set_from_file(
+            deluge.common.get_pixmap("magnet.png"))
+
         # Setup the liststore for the categories (tab pages)
         self.liststore = gtk.ListStore(int, str)
         self.treeview.set_model(self.liststore)
@@ -70,12 +73,14 @@ class Preferences(component.Component):
         # Add the default categories
         i = 0
         for category in [_("Downloads"), _("Network"), _("Bandwidth"), _("Interface"),
-            _("Other"), _("Daemon"), _("Queue"), _("Proxy"), _("Notification"), _("Plugins")]:
+            _("Other"), _("Daemon"), _("Queue"), _("Proxy"), _("Notification"),
+            _("Cache"), _("Plugins")]:
             self.liststore.append([i, category])
             i += 1
 
         # Setup plugin tab listview
         self.plugin_liststore = gtk.ListStore(str, bool)
+        self.plugin_liststore.set_sort_column_id(0, gtk.SORT_ASCENDING)
         self.plugin_listview = self.glade.get_widget("plugin_listview")
         self.plugin_listview.set_model(self.plugin_liststore)
         render = gtk.CellRendererToggle()
@@ -102,7 +107,11 @@ class Preferences(component.Component):
             "on_toggle": self.on_toggle,
             "on_test_port_clicked": self.on_test_port_clicked,
             "on_button_plugin_install_clicked": self._on_button_plugin_install_clicked,
-            "on_button_rescan_plugins_clicked": self._on_button_rescan_plugins_clicked
+            "on_button_rescan_plugins_clicked": self._on_button_rescan_plugins_clicked,
+            "on_button_find_plugins_clicked": self._on_button_find_plugins_clicked,
+            "on_button_cache_refresh_clicked": self._on_button_cache_refresh_clicked,
+            "on_combo_proxy_type_changed": self._on_combo_proxy_type_changed,
+            "on_button_associate_magnet_clicked": self._on_button_associate_magnet_clicked
         })
 
         # These get updated by requests done to the core
@@ -122,15 +131,16 @@ class Preferences(component.Component):
         label = gtk.Label()
         label.set_use_markup(True)
         label.set_markup("<b><i><big>" + name + "</big></i></b>")
-        label.set_alignment(0.05, 0.50)
-        label.set_padding(0, 10)
+        label.set_alignment(0.00, 0.50)
+        label.set_padding(10, 10)
         vbox.pack_start(label, False, True, 0)
         sep = gtk.HSeparator()
         vbox.pack_start(sep, False, True, 0)
         align = gtk.Alignment()
         align.set_padding(5, 0, 0, 0)
+        align.set(0, 0, 1, 1)
         align.add(widget)
-        vbox.pack_start(align, False, False, 0)
+        vbox.pack_start(align, True, True, 0)
         scrolled = gtk.ScrolledWindow()
         viewport = gtk.Viewport()
         viewport.set_shadow_type(gtk.SHADOW_NONE)
@@ -163,18 +173,6 @@ class Preferences(component.Component):
         if self.iter_to_remove != None:
             self.liststore.remove(self.iter_to_remove)
 
-    def _on_get_config(self, config):
-        self.core_config = config
-
-    def _on_get_available_plugins(self, plugins):
-        self.all_plugins = plugins
-
-    def _on_get_enabled_plugins(self, plugins):
-        self.enabled_plugins = plugins
-
-    def _on_get_listen_port(self, port):
-        self.active_port = port
-
     def show(self, page=None):
         """Page should be the string in the left list.. ie, 'Network' or 'Bandwidth'"""
         if page != None:
@@ -183,18 +181,37 @@ class Preferences(component.Component):
                     self.treeview.get_selection().select_path(index)
                     break
 
+        component.get("PluginManager").run_on_show_prefs()
+
         # Update the preferences dialog to reflect current config settings
         self.core_config = {}
-        try:
-            client.get_config(self._on_get_config)
-            client.get_available_plugins(self._on_get_available_plugins)
-            client.get_enabled_plugins(self._on_get_enabled_plugins)
-            client.get_listen_port(self._on_get_listen_port)
-            # Force these calls and block until we've done them all
-            client.force_call()
-        except deluge.error.NoCoreError:
-            log.debug("Not connected to a daemon..")
+        if client.connected():
+            def _on_get_config(config):
+                self.core_config = config
+                client.core.get_available_plugins().addCallback(_on_get_available_plugins)
 
+            def _on_get_available_plugins(plugins):
+                self.all_plugins = plugins
+                client.core.get_enabled_plugins().addCallback(_on_get_enabled_plugins)
+
+            def _on_get_enabled_plugins(plugins):
+                self.enabled_plugins = plugins
+                client.core.get_listen_port().addCallback(_on_get_listen_port)
+
+            def _on_get_listen_port(port):
+                self.active_port = port
+                client.core.get_cache_status().addCallback(_on_get_cache_status)
+
+            def _on_get_cache_status(status):
+                self.cache_status = status
+                self._show()
+
+            # This starts a series of client.core requests prior to showing the window
+            client.core.get_config().addCallback(_on_get_config)
+        else:
+            self._show()
+
+    def _show(self):
         if self.core_config != {} and self.core_config != None:
             core_widgets = {
                 "download_path_button": \
@@ -218,6 +235,7 @@ class Preferences(component.Component):
                 "chk_prioritize_first_last_pieces": \
                     ("active",
                         self.core_config["prioritize_first_last_pieces"]),
+                "chk_add_paused": ("active", self.core_config["add_paused"]),
                 "spin_port_min": ("value", self.core_config["listen_ports"][0]),
                 "spin_port_max": ("value", self.core_config["listen_ports"][1]),
                 "active_port_label": ("text", str(self.active_port)),
@@ -225,6 +243,7 @@ class Preferences(component.Component):
                 "spin_outgoing_port_min": ("value", self.core_config["outgoing_ports"][0]),
                 "spin_outgoing_port_max": ("value", self.core_config["outgoing_ports"][1]),
                 "chk_random_outgoing_ports": ("active", self.core_config["random_outgoing_ports"]),
+                "entry_interface": ("text", self.core_config["listen_interface"]),
                 "entry_peer_tos": ("text", self.core_config["peer_tos"]),
                 "chk_dht": ("active", self.core_config["dht"]),
                 "chk_upnp": ("active", self.core_config["upnp"]),
@@ -233,6 +252,7 @@ class Preferences(component.Component):
                 "chk_lsd": ("active", self.core_config["lsd"]),
                 "chk_new_releases": ("active", self.core_config["new_release_check"]),
                 "chk_send_info": ("active", self.core_config["send_info"]),
+                "entry_geoip": ("text", self.core_config["geoip_db_location"]),
                 "combo_encin": ("active", self.core_config["enc_in_policy"]),
                 "combo_encout": ("active", self.core_config["enc_out_policy"]),
                 "combo_enclevel": ("active", self.core_config["enc_level"]),
@@ -277,6 +297,8 @@ class Preferences(component.Component):
                 "chk_seed_ratio": ("active", self.core_config["stop_seed_at_ratio"]),
                 "spin_share_ratio": ("value", self.core_config["stop_seed_ratio"]),
                 "chk_remove_ratio": ("active", self.core_config["remove_seed_at_ratio"]),
+                "spin_cache_size": ("value", self.core_config["cache_size"]),
+                "spin_cache_expiry": ("value", self.core_config["cache_expiry"])
             }
             # Add proxy stuff
             for t in ("peer", "web_seed", "tracker", "dht"):
@@ -338,7 +360,7 @@ class Preferences(component.Component):
                 elif modifier == "not_active":
                     widget.set_active(not value)
                 elif modifier == "value":
-                    widget.set_value(value)
+                    widget.set_value(float(value))
                 elif modifier == "text":
                     widget.set_text(value)
 
@@ -358,6 +380,7 @@ class Preferences(component.Component):
                 "radio_compact_allocation",
                 "radio_full_allocation",
                 "chk_prioritize_first_last_pieces",
+                "chk_add_paused",
                 "spin_port_min",
                 "spin_port_max",
                 "active_port_label",
@@ -365,6 +388,7 @@ class Preferences(component.Component):
                 "spin_outgoing_port_min",
                 "spin_outgoing_port_max",
                 "chk_random_outgoing_ports",
+                "entry_interface",
                 "entry_peer_tos",
                 "chk_dht",
                 "chk_upnp",
@@ -373,6 +397,7 @@ class Preferences(component.Component):
                 "chk_lsd",
                 "chk_send_info",
                 "chk_new_releases",
+                "entry_geoip",
                 "combo_encin",
                 "combo_encout",
                 "combo_enclevel",
@@ -402,6 +427,10 @@ class Preferences(component.Component):
                 "spin_share_ratio_limit",
                 "spin_seed_time_ratio_limit",
                 "spin_seed_time_limit",
+                "spin_cache_size",
+                "spin_cache_expiry",
+                "button_cache_refresh",
+                "btn_testport"
             ]
             for t in ("peer", "web_seed", "tracker", "dht"):
                 core_widget_list.append("spin_proxy_port_%s" % t)
@@ -471,6 +500,10 @@ class Preferences(component.Component):
         elif self.gtkui_config["ntf_security"] == 'TLS':
             self.glade.get_widget("rad_ntf_tls").set_active(True)
 
+        ## Cache tab ##
+        if client.connected():
+            self.__update_cache_status()
+
         ## Plugins tab ##
         all_plugins = self.all_plugins
         enabled_plugins = self.enabled_plugins
@@ -486,12 +519,15 @@ class Preferences(component.Component):
             self.plugin_liststore.set_value(row, 0, plugin)
             self.plugin_liststore.set_value(row, 1, enabled)
 
-        component.get("PluginManager").run_on_show_prefs()
         # Now show the dialog
         self.pref_dialog.show()
 
-    def set_config(self):
-        """Sets all altered config values in the core"""
+    def set_config(self, hide=False):
+        """
+        Sets all altered config values in the core.
+
+        :param hide: bool, if True, will not re-show the dialog and will hide it instead
+        """
         try:
             from hashlib import sha1 as sha_hash
         except ImportError:
@@ -539,6 +575,8 @@ class Preferences(component.Component):
         new_core_config["prioritize_first_last_pieces"] = \
             self.glade.get_widget(
                 "chk_prioritize_first_last_pieces").get_active()
+        new_core_config["add_paused"] = \
+            self.glade.get_widget("chk_add_paused").get_active()
 
         ## Network tab ##
         listen_ports = []
@@ -557,7 +595,7 @@ class Preferences(component.Component):
         new_core_config["outgoing_ports"] = outgoing_ports
         new_core_config["random_outgoing_ports"] = \
             self.glade.get_widget("chk_random_outgoing_ports").get_active()
-
+        new_core_config["listen_interface"] = self.glade.get_widget("entry_interface").get_text()
         new_core_config["peer_tos"] = self.glade.get_widget("entry_peer_tos").get_text()
         new_core_config["dht"] = self.glade.get_widget("chk_dht").get_active()
         new_core_config["upnp"] = self.glade.get_widget("chk_upnp").get_active()
@@ -658,6 +696,8 @@ class Preferences(component.Component):
             self.glade.get_widget("chk_show_new_releases").get_active()
         new_core_config["send_info"] = \
             self.glade.get_widget("chk_send_info").get_active()
+        new_core_config["geoip_db_location"] = \
+            self.glade.get_widget("entry_geoip").get_text()
 
         ## Daemon tab ##
         new_core_config["daemon_port"] = \
@@ -706,6 +746,12 @@ class Preferences(component.Component):
         new_core_config["seed_time_limit"] = \
             self.glade.get_widget("spin_seed_time_limit").get_value()
 
+        ## Cache tab ##
+        new_core_config["cache_size"] = \
+            self.glade.get_widget("spin_cache_size").get_value_as_int()
+        new_core_config["cache_expiry"] = \
+            self.glade.get_widget("spin_cache_expiry").get_value_as_int()
+
         # Run plugin hook to apply preferences
         component.get("PluginManager").run_on_apply_prefs()
 
@@ -716,7 +762,7 @@ class Preferences(component.Component):
                 self.gtkui_config[key] = new_gtkui_config[key]
 
         # Core
-        if client.get_core_uri() != None:
+        if client.connected():
             # Only do this if we're connected to a daemon
             config_to_set = {}
             for key in new_core_config.keys():
@@ -725,17 +771,39 @@ class Preferences(component.Component):
                     config_to_set[key] = new_core_config[key]
 
             # Set each changed config value in the core
-            client.set_config(config_to_set)
+            client.core.set_config(config_to_set)
             client.force_call(True)
             # Update the configuration
             self.core_config.update(config_to_set)
 
-        # Re-show the dialog to make sure everything has been updated
-        self.show()
+        if hide:
+            self.hide()
+        else:
+            # Re-show the dialog to make sure everything has been updated
+            self.show()
 
     def hide(self):
         self.pref_dialog.hide()
 
+    def __update_cache_status(self):
+        # Updates the cache status labels with the info in the dict
+        for widget in self.glade.get_widget_prefix("label_cache_"):
+            key = widget.get_name()[len("label_cache_"):]
+            value = self.cache_status[key]
+            if type(value) == float:
+                value = "%.2f" % value
+            else:
+                value = str(value)
+
+            widget.set_text(value)
+
+    def _on_button_cache_refresh_clicked(self, widget):
+        def on_get_cache_status(status):
+            self.cache_status = status
+            self.__update_cache_status()
+
+        client.core.get_cache_status().addCallback(on_get_cache_status)
+
     def on_pref_dialog_delete_event(self, widget, event):
         self.hide()
         return True
@@ -783,8 +851,7 @@ class Preferences(component.Component):
 
     def on_button_ok_clicked(self, data):
         log.debug("on_button_ok_clicked")
-        self.set_config()
-        self.hide()
+        self.set_config(hide=True)
         return True
 
     def on_button_apply_clicked(self, data):
@@ -813,7 +880,7 @@ class Preferences(component.Component):
             else:
                 self.glade.get_widget("port_img").set_from_stock(gtk.STOCK_DIALOG_WARNING, 4)
                 self.glade.get_widget("port_img").show()
-        client.test_listen_port(on_get_test)
+        client.core.test_listen_port().addCallback(on_get_test)
         client.force_call()
 
     def on_plugin_toggled(self, renderer, path):
@@ -823,10 +890,9 @@ class Preferences(component.Component):
         value = self.plugin_liststore.get_value(row, 1)
         self.plugin_liststore.set_value(row, 1, not value)
         if not value:
-            client.enable_plugin(name)
-            component.get("PluginManager").enable_plugin(name)
+            client.core.enable_plugin(name)
         else:
-            client.disable_plugin(name)
+            client.core.disable_plugin(name)
             component.get("PluginManager").disable_plugin(name)
 
     def on_plugin_selection_changed(self, treeselection):
@@ -868,6 +934,7 @@ class Preferences(component.Component):
             chooser.destroy()
             return
 
+        import base64
         import shutil
         import os.path
         filename = os.path.split(filepath)[1]
@@ -879,16 +946,49 @@ class Preferences(component.Component):
 
         if not client.is_localhost():
             # We need to send this plugin to the daemon
-            client.upload_plugin(
-                filename,
-                xmlrpclib.Binary(open(filepath, "rb").read()))
+            filedump = base64.encodestring(open(filepath, "rb").read())
+            client.core.upload_plugin(filename, filedump)
 
-        client.rescan_plugins()
+        client.core.rescan_plugins()
         chooser.destroy()
         # We need to re-show the preferences dialog to show the new plugins
         self.show()
 
     def _on_button_rescan_plugins_clicked(self, widget):
         component.get("PluginManager").scan_for_plugins()
-        client.rescan_plugins()
+        if client.connected():
+            client.core.rescan_plugins()
         self.show()
+
+    def _on_button_find_plugins_clicked(self, widget):
+        deluge.common.open_url_in_browser("http://dev.deluge-torrent.org/wiki/Plugins")
+
+    def _on_combo_proxy_type_changed(self, widget):
+        name = widget.get_name().replace("combo_proxy_type_", "")
+        proxy_type = widget.get_model()[widget.get_active()][0]
+
+        prefixes = ["txt_proxy_", "label_proxy_", "spin_proxy_"]
+        hides = []
+        shows = []
+
+        if proxy_type == "None":
+            hides.extend(["password", "username", "server", "port"])
+        elif proxy_type in ("Socksv4", "Socksv5", "HTTP"):
+            hides.extend(["password", "username"])
+            shows.extend(["server", "port"])
+        elif proxy_type in ("Socksv5 W/ Auth", "HTTP W/ Auth"):
+            shows.extend(["password", "username", "server", "port"])
+
+        for h in hides:
+            for p in prefixes:
+                w = self.glade.get_widget(p + h + "_" + name)
+                if w:
+                    w.hide()
+        for s in shows:
+            for p in prefixes:
+                w = self.glade.get_widget(p + s + "_" + name)
+                if w:
+                    w.show()
+
+    def _on_button_associate_magnet_clicked(self, widget):
+        common.associate_magnet_links(True)
