@@ -31,7 +31,6 @@
 #    this exception statement from your version. If you delete this exception
 #    statement from all source files in the program, then also delete it here.
 #
-
 #
 
 
@@ -46,9 +45,11 @@ from urlparse import urlparse
 
 import deluge.common
 import deluge.component as component
-from deluge.ui.client import aclient as client
+from deluge.ui.client import client
 from deluge.log import LOG as log
-import deluge.ui.gtkui.listview as listview
+import listview
+from deluge.ui.tracker_icons import TrackerIcons
+from removetorrentdialog import RemoveTorrentDialog
 
 # Status icons.. Create them from file only once to avoid constantly
 # re-creating them.
@@ -85,6 +86,21 @@ def cell_data_statusicon(column, cell, model, row, data):
     except KeyError:
         pass
 
+def cell_data_trackericon(column, cell, model, row, data):
+    icon_path = component.get("TrackerIcons").get(model[row][data])
+    if icon_path:
+        try:
+            icon = gtk.gdk.pixbuf_new_from_file_at_size(icon_path, 16, 16)
+        except Exception, e:
+            pass
+    else:
+        icon = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, True, 8, 16, 16)
+        icon.fill(0x00000000)
+
+    if cell.get_property("pixbuf") != icon:
+        cell.set_property("pixbuf", icon)
+
+
 def cell_data_progress(column, cell, model, row, data):
     """Display progress bar with text"""
     (value, state_str) = model.get(row, *data)
@@ -118,10 +134,24 @@ def queue_column_sort(model, iter1, iter2, data):
     if v2 > v1:
         return -1
 
+def eta_column_sort(model, iter1, iter2, data):
+    v1 = model[iter1][data]
+    v2 = model[iter2][data]
+    if v1 == v2:
+        return 0
+    if v1 == 0:
+        return 1
+    if v2 == 0:
+        return -1
+    if v1 > v2:
+        return 1
+    if v2 > v1:
+        return -1
+
 class TorrentView(listview.ListView, component.Component):
     """TorrentView handles the listing of torrents."""
     def __init__(self):
-        component.Component.__init__(self, "TorrentView", interval=2000)
+        component.Component.__init__(self, "TorrentView", interval=2)
         self.window = component.get("MainWindow")
         # Call the ListView constructor
         listview.ListView.__init__(self,
@@ -175,7 +205,8 @@ class TorrentView(listview.ListView, component.Component):
         self.add_func_column(_("ETA"),
                                             listview.cell_data_time,
                                             [int],
-                                            status_field=["eta"])
+                                            status_field=["eta"],
+                                            sort_func=eta_column_sort)
         self.add_func_column(_("Ratio"),
                                             listview.cell_data_ratio,
                                             [float],
@@ -188,7 +219,8 @@ class TorrentView(listview.ListView, component.Component):
                                             listview.cell_data_date,
                                             [float],
                                             status_field=["time_added"])
-        self.add_text_column(_("Tracker"), status_field=["tracker_host"])
+        self.add_texticon_column(_("Tracker"), status_field=["tracker_host", "tracker_host"],
+                                    function=cell_data_trackericon)
 
         # Set filter to None for now
         self.filter = None
@@ -198,20 +230,32 @@ class TorrentView(listview.ListView, component.Component):
         # torrent menu popup.
         self.treeview.connect("button-press-event",
                                     self.on_button_press_event)
+        # Connect to the 'key-press-event' to know when the bring up the
+        # torrent menu popup via keypress.
+        self.treeview.connect("key-release-event", self.on_key_press_event)
         # Connect to the 'changed' event of TreeViewSelection to get selection
         # changes.
         self.treeview.get_selection().connect("changed",
                                     self.on_selection_changed)
 
         self.treeview.connect("drag-drop", self.on_drag_drop)
+        self.treeview.connect("key_press_event", self.on_key_press_event)
+
+        client.register_event_handler("TorrentStateChangedEvent", self.on_torrentstatechanged_event)
+        client.register_event_handler("TorrentAddedEvent", self.on_torrentadded_event)
+        client.register_event_handler("TorrentRemovedEvent", self.on_torrentremoved_event)
+        client.register_event_handler("SessionPausedEvent", self.on_sessionpaused_event)
+        client.register_event_handler("SessionResumedEvent", self.on_sessionresumed_event)
+        client.register_event_handler("TorrentQueueChangedEvent", self.on_torrentqueuechanged_event)
 
     def start(self):
         """Start the torrentview"""
         # We need to get the core session state to know which torrents are in
         # the session so we can add them to our list.
-        client.get_session_state(self._on_session_state)
+        client.core.get_session_state().addCallback(self._on_session_state)
 
     def _on_session_state(self, state):
+        log.debug("on_session_state: %s", state)
         self.treeview.freeze_child_notify()
         model = self.treeview.get_model()
         for torrent_id in state:
@@ -229,7 +273,14 @@ class TorrentView(listview.ListView, component.Component):
 
     def shutdown(self):
         """Called when GtkUi is exiting"""
-        self.save_state("torrentview.state")
+        if self.window.visible():
+            self.save_state()
+
+    def save_state(self):
+        """
+        Saves the state of the torrent view.
+        """
+        listview.ListView.save_state(self, "torrentview.state")
 
     def set_filter(self, filter_dict):
         """Sets filters for the torrentview..
@@ -271,8 +322,8 @@ class TorrentView(listview.ListView, component.Component):
 
         # Request the statuses for all these torrent_ids, this is async so we
         # will deal with the return in a signal callback.
-        client.get_torrents_status(
-            self._on_get_torrents_status, self.filter, status_keys)
+        client.core.get_torrents_status(
+            self.filter, status_keys).addCallback(self._on_get_torrents_status)
 
     def update(self):
         # Send a status request
@@ -309,8 +360,6 @@ class TorrentView(listview.ListView, component.Component):
                         except Exception, e:
                             log.debug("%s", e)
 
-        # Update the toolbar buttons just in case some state has changed
-        component.get("ToolBar").update_buttons()
         component.get("MenuBar").update_menu()
 
         self.prev_status = status
@@ -327,6 +376,11 @@ class TorrentView(listview.ListView, component.Component):
 
     def add_row(self, torrent_id, update=True):
         """Adds a new torrent row to the treeview"""
+        # Make sure this torrent isn't already in the list
+        for row in self.liststore:
+            if row[self.columns["torrent_id"].column_indices[0]] == torrent_id:
+                # Row already in the list
+                return
         # Insert a new row to the liststore
         row = self.liststore.append()
         # Store the torrent id
@@ -426,12 +480,64 @@ class TorrentView(listview.ListView, component.Component):
             torrentmenu.popup(None, None, None, event.button, event.time)
             return True
 
+    def on_key_press_event(self, widget, event):
+        # Menu key
+        if gtk.gdk.keyval_name(event.keyval) != "Menu":
+            return
+
+        if not self.get_selected_torrent():
+            return
+
+        torrentmenu = component.get("MenuBar").torrentmenu
+        torrentmenu.popup(None, None, None, 3, event.time)
+        return True
+
     def on_selection_changed(self, treeselection):
         """This callback is know when the selection has changed."""
         log.debug("on_selection_changed")
         component.get("TorrentDetails").update()
-        component.get("ToolBar").update_buttons()
         component.get("MenuBar").update_menu()
 
     def on_drag_drop(self, widget, drag_context, x, y, timestamp):
         widget.stop_emission("drag-drop")
+
+    def on_torrentadded_event(self, torrent_id):
+        self.add_row(torrent_id)
+        self.mark_dirty(torrent_id)
+
+    def on_torrentremoved_event(self, torrent_id):
+        self.remove_row(torrent_id)
+
+    def on_torrentstatechanged_event(self, torrent_id, state):
+        # Update the torrents state
+        for row in self.liststore:
+            if not torrent_id == row[self.columns["torrent_id"].column_indices[0]]:
+                continue
+            row[self.get_column_index(_("Progress"))[1]] = state
+
+        self.mark_dirty(torrent_id)
+
+    def on_sessionpaused_event(self):
+        self.mark_dirty()
+        self.update()
+
+    def on_sessionresumed_event(self):
+        self.mark_dirty()
+        self.update()
+
+    def on_torrentqueuechanged_event(self):
+        self.mark_dirty()
+        self.update()
+
+    # Handle keyboard shortcuts
+    def on_key_press_event(self, widget, event):
+        keyname = gtk.gdk.keyval_name(event.keyval)
+        func = getattr(self, 'keypress_' + keyname, None)
+        if func:
+            return func()
+
+    def keypress_Delete(self):
+        log.debug("keypress_Delete")
+        torrents = self.get_selected_torrents()
+        if torrents:
+            RemoveTorrentDialog(torrents).run()
